pub struct LayoutOptimizer { /* private fields */ }Expand description
Layout optimizer that chooses optimal data layouts based on device and operation type
Implementations
Sourceimpl LayoutOptimizer
impl LayoutOptimizer
Sourcepub fn preferred_layout(
&self,
device: &Device,
op_type: OperationType,
) -> DataLayout
pub fn preferred_layout( &self, device: &Device, op_type: OperationType, ) -> DataLayout
Get the preferred layout for a given device and operation type
Sourcepub fn conversion_cost(&self, from: DataLayout, to: DataLayout) -> f32
pub fn conversion_cost(&self, from: DataLayout, to: DataLayout) -> f32
Get the cost of converting between two layouts
Sourcepub fn should_convert(
&self,
from: DataLayout,
to: DataLayout,
operation_benefit: f32,
) -> bool
pub fn should_convert( &self, from: DataLayout, to: DataLayout, operation_benefit: f32, ) -> bool
Determine if a layout conversion is beneficial
Sourcepub fn auto_layout(
&self,
current_layout: DataLayout,
target_device: &Device,
op_type: OperationType,
operation_intensity: f32,
) -> DataLayout
pub fn auto_layout( &self, current_layout: DataLayout, target_device: &Device, op_type: OperationType, operation_intensity: f32, ) -> DataLayout
Auto-select the best layout for a tensor given the target device and operation