pub struct StructuredArray { /* private fields */ }Expand description
A structured array that can hold records with multiple named fields of different types
Implementations
Sourceimpl StructuredArray
impl StructuredArray
Sourcepub fn new(fields: Vec<FieldDescriptor>, len: usize) -> Self
pub fn new(fields: Vec<FieldDescriptor>, len: usize) -> Self
Create a new structured array with the given fields and capacity
Sourcepub fn with_shape(fields: Vec<FieldDescriptor>, shape: &[usize]) -> Self
pub fn with_shape(fields: Vec<FieldDescriptor>, shape: &[usize]) -> Self
Create a new multi-dimensional structured array
Sourcepub fn fields(&self) -> &[FieldDescriptor]
pub fn fields(&self) -> &[FieldDescriptor]
Get the field descriptors
Sourcepub fn field(&self, name: &str) -> Option<&FieldDescriptor>
pub fn field(&self, name: &str) -> Option<&FieldDescriptor>
Get a field descriptor by name
Sourcepub fn field_names(&self) -> Vec<&str>
pub fn field_names(&self) -> Vec<&str>
Get field names
Sourcepub fn set_field_value(
&mut self,
record_idx: usize,
field_name: &str,
value: FieldValue,
) -> Result<()>
pub fn set_field_value( &mut self, record_idx: usize, field_name: &str, value: FieldValue, ) -> Result<()>
Set a field value for a specific record
Sourcepub fn get_field_value(
&self,
record_idx: usize,
field_name: &str,
) -> Result<FieldValue>
pub fn get_field_value( &self, record_idx: usize, field_name: &str, ) -> Result<FieldValue>
Get a field value for a specific record
Sourcepub fn get_record(
&self,
record_idx: usize,
) -> Result<HashMap<String, FieldValue>>
pub fn get_record( &self, record_idx: usize, ) -> Result<HashMap<String, FieldValue>>
Get all field values for a specific record as a map
Sourcepub fn set_record(
&mut self,
record_idx: usize,
values: HashMap<String, FieldValue>,
) -> Result<()>
pub fn set_record( &mut self, record_idx: usize, values: HashMap<String, FieldValue>, ) -> Result<()>
Set all field values for a specific record
Sourcepub fn get_column(&self, field_name: &str) -> Result<Vec<FieldValue>>
pub fn get_column(&self, field_name: &str) -> Result<Vec<FieldValue>>
Extract a column (field) as a vector of values
Sourcepub fn slice(&self, start: usize, end: usize) -> Result<StructuredArray>
pub fn slice(&self, start: usize, end: usize) -> Result<StructuredArray>
Get a slice of records
Sourcepub fn from_bytes(
fields: Vec<FieldDescriptor>,
data: Vec<u8>,
len: usize,
) -> Result<Self>
pub fn from_bytes( fields: Vec<FieldDescriptor>, data: Vec<u8>, len: usize, ) -> Result<Self>
Create from raw bytes (unsafe - no validation)